@@ -1,322 +1,3 @@
-new.ctmm <- methods::setClass("ctmm", representation(info="list"), contains="list")
-new.covm <- methods::setClass("covm", representation(par="numeric",isotropic="logical"), contains="matrix")
-# try putting contains as unnamed first element of representation?
-
-#######################################
-# convenience wrapper for new.ctmm
-ctmm <- function(tau=NULL,isotropic=FALSE,range=TRUE,circle=FALSE,CPF=FALSE,error=FALSE,...)
-{
-  tau.names <- c("position","velocity","acceleration")
-  dim.names <- c("x","y")
-  List <- list(...)
-  List <- List[!sapply(List,is.null)]
-  
-  info <- List$info
-  List$info <- NULL
-  if(is.null(info)) { info=list() }
-  
-  List$error <- error
-  List$circle <- circle
-  
-  # put covariance into universal format
-  if(!is.null(List$sigma)) { List$sigma <- covm(List$sigma,isotropic=isotropic) }
-  List$isotropic <- isotropic
-  
-  # label spatial elements
-  # FIX THIS
-  #if(!is.null(List$mu)) { List$mu <- as.numeric(List$mu) ; names(List$mu) <- dim.names }
-  #if(!is.null(List$COV.mu)) { dimnames(List$COV.mu) <- list(dim.names,dim.names) }
-  
-  # label tau elements
-  K <- length(tau)
-  if(K && !CPF)
-  {
-    tau <- sort(tau,decreasing=TRUE)
-    names(tau) <- tau.names[1:K]
-    tau <- tau[tau>0]
-  }
-  if(!length(tau)) { tau <- NULL } # NULL, NA, integer(0), ...
-  List$CPF <- CPF
-  if(CPF && length(tau)) { names(tau) <- c("period","decay") }
-  List$tau <- tau
-  
-  # label parameter covariance matrix
-  cov.names <- c("area",names(tau))
-  if(circle) { cov.names <- c(cov.names,"circle") }
-  # List$COV can resolve to List$COV.mu apparently
-  if(!is.null(List[["COV"]])) { dimnames(List$COV) <- list(cov.names,cov.names) }
-  
-  if(length(tau))
-  {
-    if(tau[1]==Inf)
-    { range <- FALSE }
-    else
-    { range <- TRUE }
-  }  
-  List$range <- range
-
-  if(!range & circle) { stop("Inconsistent model options: range=FALSE, circle=TRUE.") }
-  
-  # default mean function
-  if(is.null(List$mean)) { List$mean <- "stationary" }
-  else if(List$mean=="periodic")
-  {
-    P <- attr(List$mean,"par")$P
-    if(is.null(P)) { attr(List$mean,"par")$P <- 24*60^2 }
-    # default k via Nyquist needs data
-  }
-  
-  # name mean parameters
-  if(!is.null(List$mu))
-  {
-    # format simple-style stationary mean properly
-    List$mu <- rbind(List$mu)
-    colnames(List$mu) <- c("x","y")
-    
-    if(List$mean=="periodic")
-    {
-       k <- (nrow(List$mu)-1)/2
-       K <- 1:k
-       K <- c( paste("cos",K) , paste("sin",K) )
-       K <- K[order(sequence(c(k,k)))]
-       K <- c("const",K)
-       rownames(List$mu) <- K
-    }
-    
-    if(List$mean=="polynomial")
-    {
-      k <- nrow(List$mu)-1
-      K <- 0:k
-      rownames(List$mu) <- K  
-    }
-
-    # List$COV can resolve to List$COV.mu apparently
-    if(!is.null(List[["COV.mu"]]))
-    {
-      if(length(dim(List$DOF.mu))==2)
-      { dimnames(List$DOF.mu) <- list(dim.names,dim.names) }
-      else if(length(dim(List$DOF.mu))==4)
-      { dimnames(List$DOF.mu) <- list(dim.names,K,K,dim.names) }
-      
-      if(length(dim(List$COV.mu))==2)
-      { dimnames(List$COV.mu) <- list(dim.names,dim.names) }
-      else
-      { dimnames(List$COV.mu) <- list(dim.names,K,K,dim.names) }
-    }
-  }
-  
-  result <- new.ctmm(List,info=info)
-  
-  return(result)
-}
-#print.ctmm <- function(x,...) { print.listof(x,...) }
-
-ctmm.ctmm <- function(CTMM)
-{
-  List <- methods::getDataPart(CTMM)
-  names(List) <- names(CTMM) # bug workaround
-  List$info <- attr(CTMM,"info")
-  CTMM <- do.call(ctmm,List)
-  return(CTMM)
-}
-
-# 2D covariance matrix universal format
-covm <- function(pars,isotropic=FALSE)
-{
-  if(is.null(pars))
-  { return(NULL) }
-  else if(class(pars)=="covm")
-  { return(pars) }
-  else if(length(pars)==1)
-  {
-    pars <- c(pars,0,0)
-    sigma <- diag(pars[1],2)
-  }
-  else if(length(pars)==3)
-  { sigma <- sigma.construct(pars) }
-  else if(length(pars)==4)
-  {
-    sigma <- pars
-    pars <- sigma.destruct(sigma)
-  }
-  
-  # isotropic error check
-  if(isotropic)
-  {
-    pars <- c(mean(diag(sigma)),0,0)
-    sigma <- diag(pars[1],2)
-  }
-  
-  name <- c("x","y")
-  dimnames(sigma) <- list(name,name)
-  
-  names(pars) <- c("area","eccentricity","angle")
-  
-  new.covm(sigma,par=pars,isotropic=isotropic)
-}
-
-# construct covariance matrix from 1-3 parameters
-sigma.construct <- function(pars)
-{
-  GM <- pars[1]
-  if(length(pars)==1)
-  {
-    e <- 0
-    theta <- 0
-  }
-  else
-  {
-    e <- pars[2]
-    theta <- pars[3]
-  }
-  
-  u <- c(cos(theta),sin(theta))
-  v <- c(-sin(theta),cos(theta))
-  e <- exp(e/2)
-  
-  sigma <- GM * ( (u%o%u)*e + (v%o%v)/e )
-  
-  return(sigma)
-} 
-
-# reduce covariance matrix to 1-3 parameters
-sigma.destruct <- function(sigma)
-{
-  stuff <- eigen(sigma)
-  
-  e <- stuff$values
-  GM <- sqrt(prod(e))
-  e <- log(e[1]/e[2])
-  
-  if(e==0)
-  { theta <- 0 }
-  else
-  {
-    theta <- stuff$vectors[,1]
-    theta <- atan(theta[2]/theta[1])
-  } 
-  
-  return(c(GM,e,theta))
-}
-
-# blank generic function
-pars <- function(...) { return(NA) }
-
-# return the canonical parameters of a covariance matrix
-pars.covm <- function(COVM)
-{
-  if(COVM@isotropic)
-  { return(COVM@par[1]) }
-  else
-  { return(COVM@par) }
-}
-
-# returns the canonical parameters of a tau vector
-pars.tauv <- function(tau)
-{
-  if(length(tau)==0)
-  { return(tau) }
-  else if(tau[1] < Inf)
-  { return(tau) }
-  else if(length(tau)==1)
-  { return(NULL) }
-  else
-  { return(tau[-1]) }
-}
-
-############################
-# coarce infinite parameters into finite parameters appropriate for numerics
-###########################
-ctmm.prepare <- function(data,CTMM)
-{
-  K <- length(CTMM$tau)  # dimension of hidden state per spatial dimension
-  
-  range <- TRUE
-  
-  if(K>0)
-  { 
-    # numerical limit for rangeless processes
-    if(CTMM$tau[1]==Inf)
-    {
-      range <- FALSE
-      CTMM$tau[1] <- log(2^(52/2))*(last(data$t)-data$t[1])
-      
-      # diffusion -> variance
-      if(!is.null(CTMM$sigma))
-      { 
-        T <- (CTMM$tau[1]-if(K>1){CTMM$tau[2]}else{0})
-        CTMM$sigma <- CTMM$sigma*T
-        CTMM$sigma@par[1] <- CTMM$sigma@par[1]*T
-      }
-    }
-    
-    # continuity reduction
-    CTMM$tau = CTMM$tau[CTMM$tau!=0]
-    K <- length(CTMM$tau)
-  }
-  # I am this lazy
-  if(K==0) { K <- 1 ; CTMM$tau <- 0 }
-  
-  CTMM$range <- range
-  
-  # evaluate mean function for this data set if no vector is provided
-  if(is.null(CTMM$mean.vec))
-  { CTMM$mean.vec <- call.mean(CTMM,data$t) }
-  
-  return(CTMM)
-}
-
-ctmm.repair <- function(CTMM)
-{
-  if(!CTMM$range)
-  {
-    K <- length(CTMM$tau)
-    
-    # variance -> diffusion
-    T <- (CTMM$tau[1]-if(K>1){CTMM$tau[2]}else{0})
-    CTMM$sigma <- CTMM$sigma/T
-    CTMM$sigma@par[1] <- CTMM$sigma@par[1]/T
-    
-    CTMM$tau[1] <- Inf
-    
-    # delete garbate estimates
-    CTMM$mu <- NULL
-    CTMM$COV.mu <- NULL
-    CTMM$DOF.mu <- NULL
-  }
-  
-  # erase evaluated mean vector from ctmm.prepare
-  CTMM$mean.vec <- NULL
-  
-  return(CTMM)
-}
-
-## prepare error array
-error.prepare <- function(DATA,CTMM)
-{
-  n <- length(DATA$t)
-  
-  # model the error
-  if(CTMM$error>0)
-  {
-    # is the data supplied with error estimates
-    error <- DATA$error
-    # if not, then use the modeled error
-    if(is.null(error)) { error <- rep(as.numeric(CTMM$error),n) }
-  }
-  else
-  { error <- rep(0,n) }
-  
-  return(error)
-}
-
-# degree of continuity in the model
-continuity <- function(CTMM)
-{
-  K <- sum(CTMM$tau > 0)
-  return(K)
-}
-
 ###############################
 # Propagator/Green's function and Two-time correlation from Langevin equation for Kalman filter and simulations
 langevin <- function(dt,CTMM)
@@ -347,9 +28,16 @@ langevin <- function(dt,CTMM)
       f <- f[2]
       Omega2 <- f^2 + nu^2
     }
-    else
+    else if(tau[1] != tau[2])
     { Omega2 <- 1/prod(tau) }
-
+    else # equal timescale case
+    {
+      CPF <- TRUE
+      nu <- 0
+      f <- mean(f)
+      Omega2 <- f^2
+    }
+    
     T <- 2*mean(f)/Omega2
     
     if(dt==Inf) # make this numerically relative in future
@@ -391,7 +79,7 @@ langevin <- function(dt,CTMM)
 # Kalman filter/smoother for matrix P operator and multiple mean functions
 # this is a lower level function
 # more for generalizability/readability than speed at this point
-kalman <- function(z,u,dt,CTMM,error=rep(0,nrow(z)),smooth=FALSE,sample=FALSE)
+kalman <- function(z,u,dt,CTMM,error=rep(0,nrow(z)),smooth=FALSE,sample=FALSE,weight=FALSE,residual=FALSE)
 {
   n <- nrow(z)
   DATA <- 1:ncol(z)
@@ -426,11 +114,23 @@ kalman <- function(z,u,dt,CTMM,error=rep(0,nrow(z)),smooth=FALSE,sample=FALSE)
   zCon <- array(0,c(n,K,VEC))
   sCon <- array(0,c(n,K,K))
   
-  # initial state info
-  Langevin <- langevin(dt=dt[1],CTMM=CTMM)
-  Green <- Langevin$Green
-  Sigma <- Langevin$Sigma
-  sFor[1,,] <- Sigma
+  # propagation information
+  Green <- array(0,c(n,K,K))
+  Sigma <- array(0,c(n,K,K))
+  
+  # Propagators from Langevin equation
+  for(i in 1:n)
+  {
+    # does the time lag change values? Then update the propagators.
+    if(i==1 || dt[i] != dt[i-1])
+    { Langevin <- langevin(dt=dt[i],CTMM=CTMM) }
+
+    Green[i,,] <- Langevin$Green
+    Sigma[i,,] <- Langevin$Sigma
+  }
+  
+  # first zForcast is properly zeroed
+  sFor[1,,] <- Sigma[1,,]
   
   for(i in 1:n)
   {
@@ -451,19 +151,15 @@ kalman <- function(z,u,dt,CTMM,error=rep(0,nrow(z)),smooth=FALSE,sample=FALSE)
     # update forcast estimates for next iteration
     if(i<n)
     {
-      # does the time lag change values? Then update the propagators.
-      if(dt[i+1] != dt[i])
-      {
-        Langevin <- langevin(dt=dt[i+1],CTMM=CTMM)
-        Green <- Langevin$Green
-        Sigma <- Langevin$Sigma
-      }
       #update forcast estimates now
-      zFor[i+1,,] <- Green %*% zCon[i,,]
-      sFor[i+1,,] <- ((Green %*% sCon[i,,] %*% t(Green)) + Sigma)
+      zFor[i+1,,] <- Green[i+1,,] %*% zCon[i,,]
+      sFor[i+1,,] <- ((Green[i+1,,] %*% sCon[i,,] %*% t(Green[i+1,,])) + Sigma[i+1,,])
     }
   }
   
+  # return residuals of Kalman filter only
+  if(residual) { return(zRes[,1,]/sqrt(sRes[,1,1])) }
+  
   # general quadratic form, tracing over times
   # hard-code weights for location observation only
   M <- Adj(zRes[,1,]) %*% (zRes[,1,]/sRes[,1,1])
@@ -473,7 +169,7 @@ kalman <- function(z,u,dt,CTMM,error=rep(0,nrow(z)),smooth=FALSE,sample=FALSE)
   mu <- solve(W) %*% M[MEAN,DATA]
 
   # returned profiled mean
-  if(!smooth && !sample)
+  if(!smooth && !sample && !weight)
   {
     # hard coded for position observations
     sigma <- (M[DATA,DATA] - (Adj(mu) %*% W %*% mu))/n
@@ -483,6 +179,38 @@ kalman <- function(z,u,dt,CTMM,error=rep(0,nrow(z)),smooth=FALSE,sample=FALSE)
     
     return(list(mu=mu,W=W,sigma=sigma,logdet=logdet))
   }
+  
+  #########################
+  # INVERSE FILTER (DOESN'T SEEM TO WORK)
+  #########################
+  if(weight)
+  {
+    # would be standardized residuals in diagonalization
+    zRes[,1,] <- zRes[,1,]/sRes[,1,1]
+
+    # forecast estimates for zero-mean, unit-variance filters
+    zFor <- array(0,c(n,K,VEC))
+
+    # run Kalman filter loop but residual -> data instead of data -> residual
+    for(i in 1:n)
+    {
+      # forcast residuals -> fake data
+      zRes[i,,] + (t(P) %*% zFor[i,,]) -> z[i,]
+      
+      sForP <- sFor[i,,] %*% P # why do I need this?
+      if(all(abs(sRes[i,,])<Inf)){ Gain <- sForP %*% solve(sRes[i,,]) }
+      else { Gain <- sForP %*% (0*Id) } # solve() doesn't like this case...
+      
+      # concurrent estimates
+      zCon[i,,] <- zFor[i,,] + (Gain %*% zRes[i,,])
+
+      # update forcast estimates for next iteration
+      if(i<n) { zFor[i+1,,] <- Green[i+1,,] %*% zCon[i,,] }
+    }
+    
+    return(list(mu=mu,z=z))
+  }
+  
   # delete residuals
   rm(zRes,sRes)
   
@@ -508,20 +236,25 @@ kalman <- function(z,u,dt,CTMM,error=rep(0,nrow(z)),smooth=FALSE,sample=FALSE)
   zFor <- zFor[,,DATA,drop=FALSE]
   # drop=FALSE must be here for BM/OU and I don't fully understand why
   
+  #################
+  # RANDOM SAMPLER: end point
+  #################
+  if(sample)
+  {
+    zCon[n,,] <- sapply(DATA,function(d){MASS::mvrnorm(mu=zCon[n,,d],Sigma=sCon[n,,])})
+    sCon[n,,] <- array(0,c(K,K))
+  }
+  
   # upgrade concurrent estimates to Kriged estimates  
-  Green <- langevin(dt=dt[n],CTMM=CTMM)$Green
   for(i in (n-1):1)
   {
-    L <- sCon[i,,] %*% t(Green) %*% solve(sFor[i+1,,])
+    # DOUBLE CHECK THIS
+    L <- sCon[i,,] %*% t(Green[i+1,,]) %*% solve(sFor[i+1,,])
     
     # overwrite concurrent estimate with smoothed estimate
     zCon[i,,] <- zCon[i,,] + L %*% (zCon[i+1,,]-zFor[i+1,,])
     sCon[i,,] <- sCon[i,,] + L %*% (sCon[i+1,,]-sFor[i+1,,]) %*% t(L)
     
-    # next time's propagator if necessary
-    if(dt[i] != dt[i+1])
-    { Green <- langevin(dt=dt[i],CTMM=CTMM)$Green }
-    
     #################
     # RANDOM SAMPLER
     #################
@@ -551,7 +284,7 @@ kalman <- function(z,u,dt,CTMM,error=rep(0,nrow(z)),smooth=FALSE,sample=FALSE)
 ####################################
 # log likelihood function
 ####################################
-ctmm.loglike <- function(data,CTMM=ctmm(),verbose=FALSE)
+ctmm.loglike <- function(data,CTMM=ctmm(),REML=FALSE,verbose=FALSE)
 {
   n <- length(data$t)
   
@@ -589,7 +322,8 @@ ctmm.loglike <- function(data,CTMM=ctmm(),verbose=FALSE)
   z <- cbind(data$x,data$y)
   u <- CTMM$mean.vec
   M <- ncol(u) # number of linear parameters per spatial dimension
-
+  if(REML) { REML <- n/(n-M) } # now storing the covariance debias factor here
+  
   error <- error.prepare(data,CTMM)
 
   # do we need to orient the data along the major an minor axes of sigma
@@ -620,14 +354,19 @@ ctmm.loglike <- function(data,CTMM=ctmm(),verbose=FALSE)
       # profile variance if unspecified
       if(is.na(area))
       { 
-        area <- ML.area
+        # debias?
+        if(REML) { area <- REML*ML.area }
+        else { area <- ML.area }
+        
         sigma <- covm(c(area,ecc,theta),isotropic=isotropic)
       }
       
-      COV.mu <- area * solve(KALMAN$W)
+      COV.mu <- solve(KALMAN$W/area)
       DOF.mu <- KALMAN$W
       
       loglike <- -KALMAN$logdet -(n)*log(2*pi*area) - (n)*(ML.area/area)
+      
+      if(REML) { loglike <- loglike - log(det(KALMAN$W/area)) }
     }
     else
     {
@@ -640,6 +379,8 @@ ctmm.loglike <- function(data,CTMM=ctmm(),verbose=FALSE)
       DOF.mu <- area * KALMAN$W
       
       loglike <- -KALMAN$logdet - (n)*log(2*pi*1) - (n/2)*R.sigma
+      
+      if(REML) { loglike <- loglike - log(det(KALMAN$W)) }
     }
 
     loglike <- Re(loglike)
@@ -668,13 +409,23 @@ ctmm.loglike <- function(data,CTMM=ctmm(),verbose=FALSE)
     
     ML.sigma <- KALMAN$sigma
     # profile covariance if sigma unspecified
-    if(is.null(sigma)) { sigma <- covm(ML.sigma,isotropic=isotropic) }
+    if(is.null(sigma))
+    {
+      # debias?
+      if(REML) { sigma <- REML*ML.sigma }
+      else { sigma <- ML.sigma }
+      
+      sigma <- covm(sigma,isotropic=isotropic)
+    }
     
     mu <- KALMAN$mu
     COV.mu <- solve(KALMAN$W) %o% sigma
     DOF.mu <- KALMAN$W %o% diag(2)
     
     loglike <- -KALMAN$logdet -(n/2)*log(det(2*pi*sigma)) - (n/2)*sum(diag(ML.sigma %*% solve(sigma)))
+    
+    # tensor product rule for determinants
+    if(REML) { loglike <- loglike - (1/2)*log(det(KALMAN$W)^2/det(sigma)^M) }
   }
   else if(isotropic) # ONE KALMAN FILTER WITH ERROR
   {
@@ -690,6 +441,8 @@ ctmm.loglike <- function(data,CTMM=ctmm(),verbose=FALSE)
     R.sigma <- mean(diag(R.sigma))
     
     loglike <- -KALMAN$logdet -(n)*log(2*pi*1) -(n)*R.sigma
+    
+    if(REML) { loglike <- loglike - log(det(KALMAN$W)) }
   }
   else # TWO KALMAN FILTERS WITH ERROR
   {
@@ -712,6 +465,8 @@ ctmm.loglike <- function(data,CTMM=ctmm(),verbose=FALSE)
     DOF.mu <- array(c(SIGMA[1]*KALMAN1$W,diag(0,M),diag(0,M),SIGMA[2]*KALMAN2$W),c(M,M,2,2))
 
     loglike <- -(1/2)*logdet - (n)*log(2*pi*1) - (n/2)*R.sigma
+    
+    if(REML) { loglike <- loglike - (1/2)*log(det(KALMAN1$W)*det(KALMAN2$W)) }
   }
   
   # restructure indices from m,n,x,y to x,m,n,y
@@ -774,7 +529,7 @@ ctmm.loglike <- function(data,CTMM=ctmm(),verbose=FALSE)
 
 ###########################################################
 # FIT MODEL WITH LIKELIHOOD FUNCTION (convenience wrapper to optim)
-ctmm.fit <- function(data,CTMM=ctmm(),debias=TRUE,control=list(maxit=.Machine$integer.max),...)
+ctmm.fit <- function(data,CTMM=ctmm(),REML=FALSE,control=list(maxit=.Machine$integer.max),...)
 {
   # basic info
   n <- length(data$t)
@@ -796,7 +551,11 @@ ctmm.fit <- function(data,CTMM=ctmm(),debias=TRUE,control=list(maxit=.Machine$in
     
   # evaluate mean function for this data set if no vector is provided
   if(is.null(CTMM$mean.vec))
-  { CTMM$mean.vec <- call.mean(CTMM,data$t) }
+  {
+    drift <- get(CTMM$mean)
+    U <- drift(data$t,CTMM)
+    CTMM$mean.vec <- U
+  }
   
   # CAVEATS
   if(error && circle && !isotropic) { warning("error==TRUE & circle==TRUE & isotropic==FALSE distorts error circle for speed.") }
@@ -823,7 +582,8 @@ ctmm.fit <- function(data,CTMM=ctmm(),debias=TRUE,control=list(maxit=.Machine$in
   CIRCLE <- NULL
   calPARS <- function()
   {
-    NAMES <<- "sigma"
+    if(length(SIGMA)==1) { NAMES <<- "area" }
+    else if(length(SIGMA)==3) { NAMES <<- c("area","eccentricity","angle") }
     
     if(K+range==1) # IID, BM
     { TAU <<- NULL }
@@ -951,20 +711,20 @@ ctmm.fit <- function(data,CTMM=ctmm(),debias=TRUE,control=list(maxit=.Machine$in
     CTMM$sigma <- covm(sigma,isotropic=isotropic)
     CTMM$error <- error
     
-    return(-ctmm.loglike(data,CTMM))
+    return(-ctmm.loglike(data,CTMM,REML=REML))
   }
   
   # NOW OPTIMIZE
   if(PARS==0) # EXACT
   {
     # Bi-variate Gaussian || Brownian motion with zero error
-    CTMM <- ctmm.loglike(data,CTMM=CTMM,verbose=TRUE)
+    CTMM$sigma <- NULL
+    CTMM <- ctmm.loglike(data,CTMM=CTMM,REML=REML,verbose=TRUE)
     
-    if(K==0){ DOF <- n ; CTMM$tau <- NULL }
-    else { DOF <- n-1 }
-    
-    GM <- CTMM$sigma@par[1]
-    CTMM$COV <- rbind(if(isotropic) { GM^2/DOF } else { GM^2/DOF/2 })
+    # fast calculation of sigma covariance
+    # make this consistent with REML
+    COVSTUFF <- COV.covm(CTMM$sigma,n=n,k=k.mean)
+    CTMM$COV <- COVSTUFF$COV
   }
   else # all further cases require optimization
   {
@@ -1021,18 +781,20 @@ ctmm.fit <- function(data,CTMM=ctmm(),debias=TRUE,control=list(maxit=.Machine$in
     CTMM$error <- error
     
     # verbose ML information
-    CTMM <- ctmm.loglike(data,CTMM,verbose=TRUE)
+    CTMM <- ctmm.loglike(data,CTMM,REML=REML,verbose=TRUE)
     
-    # calculate area-tau uncertainty
+    # calculate full sigma-tau uncertainty numerically
     sigma <- CTMM$sigma@par
-    SIGMA <- 1
-    SIGMAV <- 1
+    SIGMA <- 1:(if(isotropic){1}else{3})
+    SIGMAV <- 1:(if(isotropic){1}else{3})
     calPARS()
     ERROR <- NULL
     calpars()
     
     hess <- numDeriv::hessian(fn,pars)
-    grad <- numDeriv::grad(fn,pars,side=sign(pars))
+    side <- sign(pars)
+    side[side==0] <- NA
+    grad <- numDeriv::grad(fn,pars,side=side)
     # robust covariance calculation
     CTMM$COV <- cov.loglike(hess,grad)
     # convert from circulation frequency to circulation period
@@ -1045,24 +807,15 @@ ctmm.fit <- function(data,CTMM=ctmm(),debias=TRUE,control=list(maxit=.Machine$in
     dimnames(CTMM$COV) <- list(NAMES,NAMES)
   }
   
-  CTMM$mean.vec <- NULL
-  
+  # these fixed terms were not included in ctmm.loglike
+  if(REML)
+  {
+    UU <- t(U) %*% U
+    CTMM$loglike <- CTMM$loglike + log(det(UU)) + ncol(U)*log(2*pi)
+  }
+
   CTMM$AICc <- (2*k-2*CTMM$loglike) + 2*k*(k+1)/(n-k-1)
 
-  # fix for range==FALSE
-  if(debias)
-  {
-    A <- n/(n-k.mean)
-    
-    sigma <- methods::getDataPart(CTMM$sigma)
-    sigma <- A * sigma
-    CTMM$sigma <- covm(sigma)
-    
-    CTMM$COV.mu <- A^2 * CTMM$COV.mu
-    CTMM$COV[1,] <- A * CTMM$COV[1,]
-    CTMM$COV[,1] <- A * CTMM$COV[,1]
-  }
-  
   return(CTMM)
 }
 
@@ -1070,7 +823,7 @@ ctmm.fit <- function(data,CTMM=ctmm(),debias=TRUE,control=list(maxit=.Machine$in
 ####################################
 # Newton-Raphson iterate to a ctmm model
 # to test if optim worked
-newton.ctmm <- function(data,CTMM)
+newton.ctmm <- function(data,CTMM,REML=REML)
 {
   tau <- CTMM$tau
   
@@ -1078,7 +831,7 @@ newton.ctmm <- function(data,CTMM)
   fn <- function(par)
   { 
     # will need to update this for telemetry error
-    return(-ctmm.loglike(data,CTMM=ctmm(tau=par)))
+    return(-ctmm.loglike(data,CTMM=ctmm(tau=par),REML=REML))
   }
   
   D <- numDeriv::grad(fn,tau)
@@ -1089,415 +842,47 @@ newton.ctmm <- function(data,CTMM)
   return(ctmm(tau=tau,info=attr(data,"info")))
 }
 
-
-###################################################
-# Calculate good CIs for other functions
-confint.ctmm <- function(model,alpha=0.05)
+####### calculate variance and variance-covaraince from acrea and eccentricity information
+area2var <- function(CTMM)
 {
-  # z-values for low, ML, high estimates
-  z <- stats::qnorm(1-alpha/2)
+  VAR <- mean(diag(CTMM$sigma))
+  COV <- CTMM$COV
   
-  tau <- model$tau
-  tau <- tau[tau<Inf]
-  K <- length(tau)
-  
-  COV <- model$COV
-  
-  par <- NULL
-  NAME <- NULL
-  
-  # timescale uncertainty: can hit 0 and Inf
-  if(K>0)
+  if(!CTMM$isotropic)
   {
-    for(k in 1:K)
+    area <- CTMM$sigma@par["area"]
+    ecc <- CTMM$sigma@par["eccentricity"]
+    
+    # convert area, eccentricity uncertainty into mean variance uncertainty
+    grad <- array( c(cosh(ecc),area*sinh(ecc),0) , c(1,3) )
+
+    P <- nrow(COV)
+    if(P>3)
     {
-      NAME <- c(NAME,paste("tau",names(tau[k])))
-      par <- rbind(par,confint.tau(tau[k],COV[k+1,k+1],z))
+      grad <- rbind( grad , array(0,c(P-3,3)) )
+      grad <- cbind( grad , rbind( rep(0,P-3) , diag(1,P-3) ) )
     }
-  }
-  
-  # circulation period
-  circle <- model$circle
-  if(circle)
-  {
-    NAME <- c(NAME,"circle")
-    par <- rbind(par,confint.tau(circle,COV["circle","circle"],z,min=-Inf))
-  }  
+    colnames(grad) <- rownames(COV)
+    rownames(grad) <- c("variance",rownames(COV)[-(1:3)])
     
-  # standard area uncertainty: chi-square
-  GM.sigma <- sqrt(det(model$sigma))
-  COV <- COV[1,1] 
-  # effective DOF derived from ML curvature
-  DOF <- 2*GM.sigma^2/COV
-  
-  NAME <- c("area",NAME)
-  par <- rbind(chisq.ci(GM.sigma,COV,alpha),par)
-  
-  rownames(par) <- NAME
-  
-  return(par)
-}
-
-# timescale uncertainty: can hit 0 and Inf
-confint.tau <- function(tau,COV,z,min=0,max=Inf)
-{
-  # tau normal for lower CI
-  CI <- tau + c(1,-1)*z*sqrt(COV)
-  
-  # lower CI of f==1/tau normal for upper CI of tau
-  CI <- c(CI, (1/tau + c(1,-1)*z*sqrt(COV/tau^4))^-1)
-
-  # take most conservative estimates
-  CI <- range(CI)
-  
-  # enforce constraints
-  CI <- c(max(CI[1],min),min(CI[2],max))
-  
-  CI <- c(CI[1],tau,CI[2])
-  return(CI)
-}
-
-summary.ctmm <- function(object,level=0.95,level.UD=0.95,...)
-{
-  CLASS <- class(object)
-  if(CLASS=="ctmm")
-  { return(summary.ctmm.single(object,level=level,level.UD=level.UD)) }
-  else if(CLASS=="list")
-  { return(summary.ctmm.list(object,level=level,level.UD=level.UD)) }
-}
-  
-
-######################################################
-# Summarize results
-summary.ctmm.single <- function(object, level=0.95, level.UD=0.95, ...)
-{
-  alpha <- 1-level
-  alpha.UD <- 1-level
-  
-  # z-values for low, ML, high estimates
-  z <- stats::qnorm(1-alpha/2)*c(-1,0,1)
-  
-  CPF <- object$CPF
-  circle <- object$circle
-  tau <- object$tau
-  if(length(tau)>0 && tau[1]==Inf) { range <- FALSE } else { range <- TRUE }
-  tau <- tau[tau<Inf]
-  K <- length(tau)
-  
-  AM.sigma <- mean(diag(object$sigma))
-  GM.sigma <- sqrt(det(object$sigma))
-  ecc <- object$sigma@par[2]
-  
-  COV <- object$COV
-  P <- nrow(COV)
-  
-  # where to store unit information
-  name <- rep("",K+1)
-  scale <- rep(1,K+1)
-  
-  par <- confint.ctmm(object,alpha=alpha)
-  
-  # standard area to home-range area
-  par[1,] <- -2*log(alpha.UD)*pi*par[1,]
-  
-  # pretty area units
-  unit.list <- unit(par[1,2],"area")
-  name[1] <- unit.list$name
-  scale[1] <- unit.list$scale
-  
-  # pretty time units
-  P <- nrow(par)
-  if(P>1)
-  {
-    for(i in 2:P)
-    {
-      unit.list <- unit(par[i,2],"time")
-      name[i] <- unit.list$name
-      scale[i] <- unit.list$scale
-    }
+    COV <- grad %*% COV %*% t(grad)
   }
   
-  # can we estimate speed?
-  if(K>1 || (!range && K>0))
-  {
-    # RMS velocity
-    if(CPF)
-    {
-      Omega2 <- sum(c(2*pi,1)^2/tau^2)
-      grad <- -2*c(2*pi,1)^2/tau^3
-    }
-    else
-    {
-      Omega2 <- 1/prod(tau)
-      grad <- -Omega2/tau
-    }
-
-    # contribution from circulation
-    omega2 <- 0
-    if(circle)
-    {
-      omega2 <- (2*pi/circle)^2
-      grad <- c(grad,-2*omega2/circle)
-    }
-    
-    # contribution from sigma
-    # GM.sigma <- cosh(ecc)*AM.sigma
-    ms <- AM.sigma*(Omega2+omega2)
-    grad <- c(cosh(ecc)*(Omega2+omega2), AM.sigma*grad)
-    var.ms <- (grad) %*% COV %*% (grad)
-    # include mean
-    MSPEED <- mspeed(object)
-    ms <- ms + MSPEED$MS
-    var.ms <- var.ms + MSPEED$VAR
-    # root mean square velocity
-    rms <- sqrt(ms)
-    var.rms <- var.ms * (1/2/rms)^2
-    
-    # pretty units
-    unit.list <- unit(rms,"speed")
-    name <- c(name,unit.list$name)
-    scale <- c(scale,unit.list$scale)
-    
-    log.rms <- log(rms)
-    var.log.rms <- var.rms * (1/rms)^2
-    
-    log.rms <- log.rms + z*sqrt(var.log.rms)
-    rms <- exp(log.rms)
-    
-    par <- rbind(par,rms)
-    rownames(par)[P+1] <- "speed"
-  }
-  
-  # Fix unit choice
-  par <- par/scale
-  
-  # affix units
-  rownames(par) <- paste(rownames(par)," (",name,")",sep="")
-  
-  colnames(par) <- c("low","ML","high")
-
-  if(!range) { par <- par[-1,] } # delete off "area" (really diffusion)
-  
-  return(par)
-}
-#methods::setMethod("summary",signature(object="ctmm"), function(object,...) summary.ctmm(object,...))
-
-
-sort.ctmm <- function(x, decreasing=FALSE, IC="AICc", ...)
-{
-  ICS <- sapply(x,function(m){m[[IC]]})
-  IND <- sort(ICS,method="quick",index.return=TRUE,decreasing=decreasing)$ix
-  x <- x[IND]
-}
-
-
-summary.ctmm.list <- function(object, IC="AICc", ...)
-{
-  object <- sort.ctmm(object,IC=IC)
-  ICS <- sapply(object,function(m){m[[IC]]})
-  ICS <- ICS - ICS[[1]]
-  ICS <- array(ICS,c(length(ICS),1))
-  rownames(ICS) <- names(object)
-  colnames(ICS) <- paste("d",IC,sep="")
-  return(ICS)
-}
-
-###################################################
-# fit a bunch of models and select the best one
-ctmm.select <- function(data,CTMM,verbose=FALSE,IC="AICc",...)
-{
-  MODELS <- list()
-
-  # listify model guesses
-  if(class(CTMM)=="ctmm") { CTMM <- list(CTMM) }
-  
-  for(i in 1:length(CTMM))
-  {  
-    if(CTMM[[i]]$CPF)
-    { MODELS <- c(MODELS,ctmm.select.cpf(data,CTMM[[i]])) }
-    else
-    { MODELS <- c(MODELS,ctmm.select.ouf(data,CTMM[[i]])) }
-  }
-  
-  # fit models
-  MODELS <- lapply(MODELS,function(M) { ctmm.fit(data,M,...) })
-  
-  # sort models by AICc
-  MODELS <- sort.ctmm(MODELS,IC=IC)
-  
-  # return everything
-  if(verbose) { return(MODELS) }
-  else { return(MODELS[[1]]) }
-}
-
-
-ctmm.select.cpf <- function(data,CTMM)
-{
-  MODELS <- list()
-  
-  MOD <- CTMM
-  MODELS <- c(MODELS,ctmm.select.isotropy(data,MOD,"CPF"))
-
-  MOD$tau <- CTMM$tau[2]
-  MODELS <- c(MODELS,ctmm.select.isotropy(data,MOD,"OU"))
-
-  MOD <- CTMM$tau <- NULL
-  MODELS <- c(MODELS,ctmm.select.isotropy(data,MOD,"IID"))
-
-  return(MODELS)  
-}
-
-
-ctmm.select.ouf <- function(data,CTMM)
-{
-  MODELS <- list()
-  NAMES <- c("IID","OU","OUF")
-
-  # step down in autocorrelation degree
-  for(K in length(CTMM$tau):0) 
-  {
-    MOD <- CTMM
-
-    # fix K
-    if(K == 0) { MOD$tau <- NULL } else { MOD$tau <- MOD$tau[1:K] }
-
-    MODELS <- c(MODELS,ctmm.select.isotropy(data,MOD,NAMES[K+1]))
-  }
-
-  return(MODELS)
-}
-
-
-ctmm.select.isotropy <- function(data,CTMM,NAME=NULL)
-{
-  MODELS <- list()
-  MOD <- CTMM
-  
-  # name model
-  if(MOD$isotropic)
-  { name <- paste(NAME,"isotropic") }
-  else
-  { name <- paste(NAME,"anisotropic") } 
-  
-  # given model
-  MODELS <- c(MODELS,ctmm.select.circle(data,MOD,name))
-  
-  # step down in isotropy
-  if(CTMM$isotropic==FALSE)
-  {
-    MOD$isotropic <- TRUE
-    MOD$sigma <- covm(MOD$sigma,isotropic=TRUE)
-    name <- paste(NAME,"isotropic")
-    MODELS <- c(MODELS,ctmm.select.circle(data,MOD,name))
-  }
-  
-  return(MODELS)
-}
-
-
-ctmm.select.circle <- function(data,CTMM,NAME=NULL)
-{
-  MODELS <- list()
-  MOD <- CTMM
-  
-  if(length(CTMM$tau)==0 || CTMM$tau==0)
-  { CTMM$circle <- FALSE }
-  
-  # given model
-  if(MOD$circle)
-  { name <- paste(NAME,"circle") }
-  else
-  { name <- NAME }
-  MODELS[[name]] <- MOD
-  
-  #simpler alternative
-  if(MOD$circle)
-  {
-    MOD$circle <- FALSE
-    MODELS[[NAME]] <- MOD
-  }
-  
-  return(MODELS)
-}
-
-
-#################################################
-# SLOW LIKELIHOOD FUNCTION
-# not to be exposed to end users
-# THIS NEEDS TO BE FIXED WITH ERRORS
-ctmm.loglike.slow <- function(data,CTMM)
-{  
-  t <- data$t
-  x <- data$x
-  y <- data$y
-
-  tau <- CTMM$tau
-  sigma <- CTMM$sigma
-  mu <- CTMM$mu
-  isotropic <- CTMM$isotropic
-  
-  K <- length(tau)
-  n <- length(t)
-  
-  # lag matrix
-  C <- outer(t,t,"-")
-  C <- abs(C)
-  
-  # now the correlation matrix
-  if(K==1)
-  { C <- exp(-C/tau) }
-  else if(K==2)
-  { C <- (tau[1]*exp(-C/tau[1])-tau[2]*exp(-C/tau[2]))/(tau[1]-tau[2]) }
-  
-  logdetC <- determinant(C,logarithm=TRUE)$modulus
-  
-  u <- rep(1,n) # stationary mean function
-  
-  # now the inverse correlation matrix times vectors
-  C <- solve(C,cbind(x,y,u))
-  
-  Cx <- C[,1]
-  Cy <- C[,2]
-  w <- C[,3]
-
-  W <- sum(w)
-  
-  if(is.null(mu)) { mu <- c(sum(Cx), sum(Cy))/W }
-  
-  # detrend mean
-  x <- x - mu[1]
-  y <- y - mu[2]
-
-  Cx <- Cx - mu[1]*w
-  Cy <- Cy - mu[2]*w
-  
-  if(is.null(sigma))
-  {
-    sigma <- x %*% Cy
-    sigma <- rbind( c(x %*% Cx, sigma) , c( sigma, y %*% Cy) )/n
-    sigma <- covm(sigma,isotropic)
-  }
-
-  if(length(sigma) == 1) { sigma <- sigma * diag(2)  }
-  
-  COV.mu <- methods::getDataPart(sigma)/W
-  
-  loglike <- -logdetC - (n/2)*log(det(2*pi*sigma)) - n
-
-  CTMM <- ctmm(loglike=loglike,tau=tau,sigma=sigma,mu=mu,COV.mu=COV.mu,DOF.mu=W,info=attr(data,"info"))
-    
-  return(CTMM)
+  return(COV)
 }
 
 
 ###################
 # general parameter guessing function
 ###################
-ctmm.guess <- function(data,variogram=NULL,CTMM=ctmm(),name="GUESS",interactive=TRUE)
+ctmm.guess <- function(data,CTMM=ctmm(),variogram=NULL,name="GUESS",interactive=TRUE)
 {
   if(is.null(variogram)) { variogram = variogram(data) }
   
-  mu <- c(mean(data$x),mean(data$y))
+  # mean specific guesswork/preparation
+  drift <- get(CTMM$mean)
+  CTMM <- drift@init(data,CTMM)
+  mu <- CTMM$mu
   
   z <- cbind(data$x - mu[1], data$y - mu[2])
   sigma <- t(z) %*% z
@@ -1509,7 +894,7 @@ ctmm.guess <- function(data,variogram=NULL,CTMM=ctmm(),name="GUESS",interactive=
   sigma <- sigma / (n-1)  
 
   CTMM$mu <- mu
-  CTMM$sigma <- covm(sigma)
+  CTMM$sigma <- covm(sigma,isotropic=CTMM$isotropic)
   
   # estimate circulation period
   if(CTMM$circle==1)
@@ -1530,3 +915,74 @@ ctmm.guess <- function(data,variogram=NULL,CTMM=ctmm(),name="GUESS",interactive=
   
   variogram.fit(variogram,CTMM=CTMM,name=name,interactive=interactive)
 }
+
+############
+residuals.ctmm <- function(object,data,...)
+{
+  object <- ctmm.prepare(data,object)
+  error <- error.prepare(data,object)
+  t <- data$t
+  dt <- c(Inf,diff(t))
+  
+  # effective DOF of variance estimate
+  n <- 2*object$sigma@par["area"]^2/object$COV["area","area"]
+  
+  # subtract off mean function
+  z <- cbind(data$x,data$y)
+  drift <- get(object$mean)
+  mu <- drift(data$t,object) %*% object$mu
+  z <- z - mu
+  
+  sigma <- object$sigma
+  area <- sigma@par["area"]
+  theta <- sigma@par["angle"]
+  ecc <- sigma@par["eccentricity"]
+
+  circle <- object$circle
+  if(circle) { circle <- 2*pi/circle }
+  
+  
+  # rotate to eigen-basis
+  z <- z %*% t(rotate(-theta))
+  
+  if(circle)
+  {
+    # proportional standardization from ellipse to circle
+    if(ecc)
+    {
+      z[,1] <- z[,1] * exp(-ecc/4)
+      z[,2] <- z[,2] * exp(+ecc/4)
+    }
+    z <- cbind(z[,1] + 1i*z[,2])
+    
+    # corotating frame
+    R <- exp(-1i*circle*(t-t[1]))
+    z <- R * z
+    
+    # preserves error volume, but distorts error ellipse
+    object$sigma <- area
+    z <- kalman(z,u=NULL,dt=dt,object,error=error,residual=TRUE)
+    
+    # de-complexify result
+    z <- cbind(Re(z),Im(z))
+  }
+  else
+  {
+    # send each axis through with correct variance and error
+    object$sigma <- area*exp(+ecc/2)
+    z[,1] <- kalman(z[,1,drop=FALSE],u=NULL,dt=dt,object,error=error,residual=TRUE)
+    
+    object$sigma <- area*exp(-ecc/2)
+    z[,2] <- kalman(z[,2,drop=FALSE],u=NULL,dt=dt,object,error=error,residual=TRUE)
+  }
+
+  data@info$UERE <- NULL
+  COV <- cbind(2/n)
+  dimnames(COV) <- list("area","area")
+  data@info$CTMM <- ctmm(mu=rbind(c(0,0)),sigma=1,isotropic=T,COV=COV)
+  data <- data[,c("t","x","y")]
+  data$x <- z[,1]
+  data$y <- z[,2]
+  
+  return(data) # check class of this object: data.frame or telemetry
+}
\ No newline at end of file
